/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action.model.validators;

import com.google.common.base.Strings;
import org.opengroup.osdu.register.action.model.Filter;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.util.List;

public class FilterValidator implements ConstraintValidator<ValidFilter, Filter> {

    private static final int MaxFilterVals = 30;

    @Override
    public void initialize(ValidFilter constraintAnnotation) {
        //needed by interface - we don't use
    }

    @Override
    public boolean isValid(Filter filter, ConstraintValidatorContext context) {
        if (!validateFilterProperty(filter.getSource(), context)) {
            return false;
        } else if (!validateFilterProperty(filter.getEntityType(), context)) {
            return false;
        } else if (!validateFilterProperty(filter.getVersion(), context)) {
            return false;
        }
        return true;
    }

    private boolean validateFilterProperty(List<String> filterProperty, ConstraintValidatorContext context) {
        if (filterProperty.isEmpty()) {
            return addError(context, "No filter value found. Must provide a filter value. Use '*' if you want to match all values for a filter property.");
        } else if (filterProperty.size() > MaxFilterVals) {
            return addError(context, String.format("Can't provide more than %s filter values for any 1 filter property.", MaxFilterVals));
        } else if (filterProperty.stream().anyMatch(s -> Strings.isNullOrEmpty(s))) {
            return addError(context, "Found empty filter values. Must provide a filter value. Use '*' if you want to match all values for a filter property.");
        } else if (filterProperty.size() > 1 && filterProperty.contains("*")) {
            return addError(context, "Can't provide wildcard '*' and specific filter values. Wildcard means you match all values for that property and must be used alone.");
        }
        return true;
    }

    boolean addError(ConstraintValidatorContext context, String error) {
        context.disableDefaultConstraintViolation();
        context.buildConstraintViolationWithTemplate(error).addConstraintViolation();
        return false;
    }
}
