/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.api;

import org.opengroup.osdu.core.common.model.notification.Topic;
import org.opengroup.osdu.register.api.dto.SubscriptionInfo;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.subscriber.model.Secret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ISubscriptionRepository;
import org.opengroup.osdu.register.subscriber.services.AvailableTopics;
import org.opengroup.osdu.register.subscriber.services.CreateSubscription;
import org.opengroup.osdu.register.subscriber.services.DeleteSubscription;
import org.opengroup.osdu.register.subscriber.services.UpdateSubscriptionSecret;
import org.opengroup.osdu.register.utils.ServiceRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.annotation.RequestScope;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;

@RestController
@RequestMapping("/")
@RequestScope
@Validated
public class SubscriberApi {

    @Inject
    private Provider<CreateSubscription> createSubscription; //lazy load for performance
    @Inject
    private Provider<DeleteSubscription> deleteSubscription; //lazy load for performance
    @Inject
    private Provider<UpdateSubscriptionSecret> updateSubscriptionSecret; //lazy load for performance
    @Inject
    private Provider<ISubscriptionRepository> subscriptionRepository;
    @Autowired
    private AvailableTopics topics;

    @PostMapping("subscription")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "')")
    public ResponseEntity<Subscription> createSubscription(@RequestBody @Valid Subscription input) throws Exception {
        input = createSubscription.get().run(input);
        return new ResponseEntity<>(input, HttpStatus.CREATED);
    }

    @GetMapping("subscription")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "')")
    public ResponseEntity<List<Subscription>> querySubscription(@RequestParam("notificationId") String notificationId) {
        List<Subscription> output = subscriptionRepository.get().query(notificationId);
        return new ResponseEntity<>(output, HttpStatus.OK);
    }

    @GetMapping("subscription/{id}")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "')")
    public ResponseEntity<SubscriptionInfo> getSubscription(@PathVariable("id") @NotBlank String id) {
        Subscription output = subscriptionRepository.get().get(id);
        return new ResponseEntity<>(new SubscriptionInfo(output), HttpStatus.OK);
    }

    @PutMapping("subscription/{id}/secret")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "')")
    public ResponseEntity updateSecret(@RequestBody @Valid Secret input, @PathVariable("id") @NotBlank String id) {
        if (updateSubscriptionSecret.get().run(input, id)) {
            return new ResponseEntity(HttpStatus.NO_CONTENT);
        } else {
            throw new AppException(org.apache.http.HttpStatus.SC_NOT_FOUND, "Not found",
                    String.format("Could not update subscription with id %s because it could not be found", id));
        }
    }

    @DeleteMapping("subscription/{id}")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "')")
    public ResponseEntity deleteSubscription(@PathVariable("id") @NotBlank String id) {
        if (deleteSubscription.get().run(id))
            return new ResponseEntity(HttpStatus.NO_CONTENT);
        else
            throw new AppException(org.apache.http.HttpStatus.SC_NOT_FOUND, "Not found",
                    String.format("Could not delete subscription with id %s because it could not be found", id));
    }

    @GetMapping("topics")
    @PreAuthorize("@authorizationFilter.hasAnyPermission('" + ServiceRole.OPS + "', '" + ServiceRole.ADMIN + "', '" + ServiceRole.EDITOR + "')")
    public ResponseEntity<List<Topic>> listMessages() {
        return new ResponseEntity<>(topics.list(), HttpStatus.OK);
    }
}
