/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber.services;

import com.google.common.hash.Hashing;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import org.opengroup.osdu.core.common.cryptographic.ISignatureService;
import org.opengroup.osdu.core.common.http.HttpRequest;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.http.IHttpClient;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.register.subscriber.model.GsaSecret;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Secret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.utils.AppServiceConfig;
import org.opengroup.osdu.register.utils.Constants;
import org.opengroup.osdu.register.utils.IGoogleServiceAccount;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Map;
import java.util.UUID;

@Service
public class ChallengeResponseCheck {

    @Autowired
    private JaxRsDpsLog log;
    @Autowired
    private DpsHeaders dpsHeaders;
    @Autowired
    private ISignatureService signatureService;
    @Autowired
    private IGoogleServiceAccount googleServiceAccount;
    @Autowired
    private AppServiceConfig serviceConfig;
    @Autowired
    private IHttpClient httpClient;

    private final static int timeOutMillis = 3000;

    public boolean runCreate(Subscription input) {
        return run(input, input.getSecret());
    }

    public boolean runUpdate(Subscription input, Secret newSecret) {
        return run(input, newSecret);
    }

    private boolean run(Subscription input, Secret secret) {
        try {
            if (!serviceConfig.isLocalEnvironment()) {
                if (!input.getPushEndpoint().startsWith("https://")) {
                    log.error("Challenge response check requires 'https' endpoint.");
                    return false;
                }
            }

            // Update the secret before challenging
            input.setSecret(secret);
            String crc = getCrc();
            String secretString;
            HttpResponse response;
            // If GSA Secret type, add authorization header using google id token
            if (secret.getSecretType().equalsIgnoreCase(Constants.GSASecret)) {
                GsaSecret gsaSecret = (GsaSecret) secret;

                // Generate token
                JsonParser jsonParser = new JsonParser();
                JsonElement root = jsonParser.parse(gsaSecret.getValue().getKey());
                String keyString = root.getAsJsonObject().toString();
                String idToken = this.googleServiceAccount.getIdToken(keyString, gsaSecret.getValue().getAudience());
                secretString = this.googleServiceAccount.getPrivateKeyId(keyString);

                // send gsa challange with idtoken in header
                response = sendGsaChallenge(input, crc, idToken);
            } else {
                secretString = ((HmacSecret) secret).getValue();
                String hmacSignature = this.signatureService.getSignedSignature(input.getPushEndpoint(), secretString);
                response = sendHmacChallenge(input, crc, hmacSignature);
            }

            String returnedHash = getChallengeResponseHash(response);
            String encodedHash = getExpectedResponse(secretString, crc);
            return encodedHash.equalsIgnoreCase(returnedHash);
        } catch (Exception e) {
            log.error("Unexpected error during challenge response check.", e);
            return false;
        }
    }

    protected String getCrc() {
        return UUID.randomUUID().toString();
    }

    private HttpResponse sendHmacChallenge(Subscription input, String crc, String hmacSignature) {
        HttpRequest request = HttpRequest
                .get()
                .connectionTimeout(timeOutMillis)
                .url(input.getPushEndpoint() + "?crc=" + crc + "&hmac=" + hmacSignature)
                .headers(dpsHeaders.getHeaders())
                .build();

        return httpClient.send(request);
    }

    private HttpResponse sendGsaChallenge(Subscription input, String crc, String authHeader) {
        Map<String, String> headers = dpsHeaders.getHeaders();
        headers.put(DpsHeaders.AUTHORIZATION, authHeader);
        HttpRequest request = HttpRequest
                .get()
                .connectionTimeout(timeOutMillis)
                .url(input.getPushEndpoint() + "?crc=" + crc)
                .headers(headers)
                .build();

        return httpClient.send(request);
    }

    private String getExpectedResponse(String secret, String crc) {
        String combined = secret + crc;
        String hash = Hashing.sha256()
                .hashString(combined, StandardCharsets.UTF_8)
                .toString();
        return Base64.getEncoder().encodeToString(hash.getBytes());
    }

    private String getChallengeResponseHash(HttpResponse response) {
        log.info("Challenge info: " + response.toString());
        if (!response.isSuccessCode()) {
            return "";
        }

        ChallengeResponse body = response.parseBody(ChallengeResponse.class);
        return body.responseHash;
    }

    class ChallengeResponse {
        public String responseHash = "";
    }
}
