/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action.services;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.opengroup.osdu.register.action.model.Action;
import org.opengroup.osdu.register.action.model.Filter;
import org.opengroup.osdu.register.action.model.ParsedAction;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class RetreiveActionServiceTest {

    @Mock
    private TestActionService actionService;

    @Mock
    private JaxRsDpsLog logger;

    @InjectMocks
    private RetrieveActionServiceImpl retrieveActionService;

    private List<Action> actions;

    @Test
    public void shouldIgnoreActionWithNoParsedErrors() {
        Filter filter = getCommonTestFilter();
        actions = new ArrayList<>();

        Action actionWithOnePattern = getTestActionWithUrl("https://myapp.osdu.opengroup.org/action/{id}", filter);
        actions.add(actionWithOnePattern);

        String jsonString = "{\n" +
                "   \"id\": \"common:regularheightfield:123456\",\n" +
                "   \"kind\": \"common:fakepetrel:regularheightfield:1.0.0\",\n" +
                "   \"data\": {\n" +
                "     \"msg\": \"Hello\"\n" +
                "   }\n" +
                "}";

        JsonNode actualObj = getJsonNode(jsonString);
        when(this.actionService.validateRegex(any())).thenReturn(ParsedAction.builder().url("https://myapp.osdu.opengroup.org/action/common:regularheightfield:123456").build());
        List<Action> outputActions = retrieveActionService.getActions(actions, actualObj);
        Assert.assertEquals(1, outputActions.size());
    }


    @Test
    public void shouldIgnoreActionWithParsedErrors() {
        Filter filter = getCommonTestFilter();
        actions = new ArrayList<>();

        Action actionWithOnePattern = getTestActionWithUrl("https://myapp.osdu.opengroup.org/action/{id}", filter);
        actions.add(actionWithOnePattern);

        String jsonString = "{\n" +
                "   \"id\": \"common:regularheightfield:123456\",\n" +
                "   \"kind\": \"common:fakepetrel:regularheightfield:1.0.0\",\n" +
                "   \"data\": {\n" +
                "     \"msg\": \"Hello\"\n" +
                "   }\n" +
                "}";

        JsonNode actualObj = getJsonNode(jsonString);
        ParsedAction parsedAction = new ParsedAction();
        parsedAction.setErrors("Error");
        when(this.actionService.validateRegex(any())).thenReturn(parsedAction);
        List<Action> outputActions = retrieveActionService.getActions(actions, actualObj);
        Assert.assertEquals(0, outputActions.size());
    }

    private Action getTestActionWithUrl(String url, Filter filter) {
        Action action = new Action();
        action.setUrl(url);
        action.setFilter(filter);
        return action;
    }

    private Filter getCommonTestFilter() {
        List<String> types = new ArrayList<>(Arrays.asList("regularheightfield", "petrel"));
        List<String> sources = new ArrayList<>(Arrays.asList("petrel"));
        List<String> versions = new ArrayList<>(Arrays.asList("1.0.0"));
        Filter filter = new Filter();
        filter.setEntityType(types);
        filter.setSource(sources);
        filter.setVersion(versions);
        return filter;
    }

    private JsonNode getJsonNode(String json) {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.readTree(json);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
}
