/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.api;

import com.fasterxml.jackson.databind.JsonNode;
import org.apache.http.HttpStatus;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.opengroup.osdu.register.action.model.Action;
import org.opengroup.osdu.register.action.model.CreateActionDto;
import org.opengroup.osdu.register.action.model.ParsedAction;
import org.opengroup.osdu.register.action.model.TestActionRequest;
import org.opengroup.osdu.register.provider.interfaces.action.IActionRepo;
import org.opengroup.osdu.register.provider.interfaces.action.IRetrieveActionService;
import org.opengroup.osdu.register.action.services.TestActionService;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.logging.AuditLog;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.springframework.http.ResponseEntity;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ActionApiTest {

    @Mock
    private IActionRepo repo;
    @Mock
    private AuditLog auditLog;
    @Mock
    private JaxRsDpsLog log;
    @Mock
    private IRetrieveActionService retrieveActionService;
    @Mock
    private JsonNode jsonNode;
    @Mock
    private TestActionService testActionService;
    @InjectMocks
    private ActionApi sut;

    @Test
    public void should_return201WithAction_when_creatingWithAnActionDto() {
        Action output = new Action();
        when(repo.createAction(any())).thenReturn(output);

        ResponseEntity response = sut.createAction(new CreateActionDto());

        assertEquals(HttpStatus.SC_CREATED, response.getStatusCodeValue());
        assertEquals(output, response.getBody());
    }

    @Test
    public void should_return200_getAction_when_givenValidId() {
        ResponseEntity response = this.sut.getAction(any());
        assertEquals(HttpStatus.SC_OK, response.getStatusCodeValue());
    }

    @Test
    public void should_return204_deleteAction_when_givenValidId() {
        String id = "1560392249745";
        when(this.repo.delete(id)).thenReturn(true);

        ResponseEntity response = this.sut.deleteAction(id);

        assertEquals(HttpStatus.SC_NO_CONTENT, response.getStatusCodeValue());
    }

    @Test
    public void should_return200_when_retrievingAnAction() {
        List<Action> actions = new ArrayList<>();
        when(repo.getAllActions()).thenReturn(actions);
        when(retrieveActionService.getActions(any(), any())).thenReturn(actions);

        ResponseEntity response = sut.retrieveAction(jsonNode);

        assertEquals(200, response.getStatusCodeValue());
        assertEquals(actions, response.getBody());
    }

    @Test
    public void should_return500_when_retrievingAnAction_output_throwOtherTypeException() {
        List<Action> actions = new ArrayList<>();
        when(repo.getAllActions()).thenReturn(actions);
        RuntimeException e = new RuntimeException("other exceptions");
        when(retrieveActionService.getActions(any(), any())).thenThrow(e);

        ResponseEntity response = sut.retrieveAction(jsonNode);

        assertEquals(500, response.getStatusCodeValue());
        verify(this.log).error("retrieve action failed", e);
    }

    @Test
    public void should_return200_getAction() {
        ResponseEntity response = this.sut.getAction(any());

        assertEquals(HttpStatus.SC_OK, response.getStatusCodeValue());
    }

    @Test
    public void should_return204_deleteAction() {
        String id = "123";

        when(repo.delete(any())).thenReturn(true);
        ResponseEntity response = this.sut.deleteAction(id);

        assertEquals(HttpStatus.SC_NO_CONTENT, response.getStatusCodeValue());
    }

    @Test(expected = AppException.class)
    public void should_return404_deleteAction() {
        String id = "12345";
        when(this.repo.delete(any())).thenReturn(false);

        ResponseEntity response = this.sut.deleteAction(id);
    }

    @Test
    public void should_return404_deleteAction_when_givenInvalidId() {
        String id = "1560392249745";
        when(this.repo.delete(id)).thenReturn(false);

        try {
            this.sut.deleteAction(id);
            fail("Should not succeed!");
        } catch (AppException e) {
            assertEquals(HttpStatus.SC_NOT_FOUND, e.getError().getCode());
            assertEquals(String.format("Could not delete Action with id %s because it could not be found", id), e.getError().getMessage());
            assertEquals("Not found", e.getError().getReason());
        } catch (Exception e) {
            fail("Should not throw this exception " + e.getMessage());
        }
    }

    @Test
    public void should_return200WithAction_when_creatingWithAnActionPostDto() {
        ParsedAction output = new ParsedAction();
        output.setErrors("");
        when(testActionService.validateRegex(any())).thenReturn(output);
        ParsedAction response = testActionService.validateRegex(new TestActionRequest());
        assertEquals("", response.getErrors());
    }

    @Test
    public void should_return400WithAction_when_creatingWithAnActionPostDto() {
        ParsedAction output = new ParsedAction();
        output.setErrors("Invalid regex");
        when(testActionService.validateRegex(any())).thenReturn(output);
        ParsedAction response = testActionService.validateRegex(new TestActionRequest());
        assertEquals("Invalid regex", response.getErrors());
    }
}
