/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.api;

import org.apache.http.HttpStatus;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.model.notification.Topic;
import org.opengroup.osdu.register.api.dto.SubscriptionInfo;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.logging.AuditLog;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ISubscriptionRepository;
import org.opengroup.osdu.register.subscriber.services.AvailableTopics;
import org.opengroup.osdu.register.subscriber.services.CreateSubscription;
import org.opengroup.osdu.register.subscriber.services.DeleteSubscription;
import org.opengroup.osdu.register.subscriber.services.UpdateSubscriptionSecret;
import org.springframework.http.ResponseEntity;

import javax.inject.Provider;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class SubscriberApiTest {

    @Mock
    private Provider<CreateSubscription> createSubscription;
    @Mock
    private Provider<DeleteSubscription> deleteSubscription;
    @Mock
    private Provider<UpdateSubscriptionSecret> updateSubscriptionSecret;
    @Mock
    private Provider<ISubscriptionRepository> subscriptionRepository;
    @Mock
    private AvailableTopics topics;
    @Mock
    private AuditLog auditLog;
    @Mock
    private CreateSubscription cs;
    @Mock
    private DeleteSubscription ds;
    @Mock
    private UpdateSubscriptionSecret uss;
    @Mock
    private ISubscriptionRepository sr;
    @InjectMocks
    private SubscriberApi sut;

    @Test
    public void should_return201_when_givenCreatingSubscriberWithValidInput() throws Exception {
        Subscription input = new Subscription();
        when(cs.run(input)).thenReturn(input);
        when(createSubscription.get()).thenReturn(cs);

        ResponseEntity response = this.sut.createSubscription(input);

        assertEquals(HttpStatus.SC_CREATED, response.getStatusCodeValue());
        assertEquals(input, response.getBody());
    }

    @Test
    public void should_return200_when_retrievingSubscriptionWithValidId() {
        Subscription input = new Subscription();
        SubscriptionInfo subsInfo = new SubscriptionInfo(input);
        when(subscriptionRepository.get()).thenReturn(sr);
        when(sr.get("1")).thenReturn(input);

        ResponseEntity response = this.sut.getSubscription("1");

        assertEquals(HttpStatus.SC_OK, response.getStatusCodeValue());
        assertEquals(subsInfo, response.getBody());
    }

    @Test
    public void should_return204_when_deletingSubscriptionWithValidId() {
        when(deleteSubscription.get()).thenReturn(ds);
        when(ds.run("1")).thenReturn(true);

        ResponseEntity response = this.sut.deleteSubscription("1");

        assertEquals(HttpStatus.SC_NO_CONTENT, response.getStatusCodeValue());
        assertNull(response.getBody());
    }

    @Test
    public void should_throw404_when_deletingSubscriptionThatDoesNotExist() {
        when(deleteSubscription.get()).thenReturn(ds);
        try {
            this.sut.deleteSubscription("1");
            fail("Should not succeed!");
        } catch (AppException e) {
            assertEquals(404, e.getError().getCode());
            assertEquals("Could not delete subscription with id 1 because it could not be found", e.getError().getMessage());
            assertEquals("Not found", e.getError().getReason());
        }
        verify(this.auditLog, times(0)).deleteSubscriptionSuccess("1");
    }

    @Test
    public void should_return200_when_listingMessages() {
        List<Topic> input = new ArrayList<>();
        when(topics.list()).thenReturn(input);

        ResponseEntity response = this.sut.listMessages();

        assertEquals(HttpStatus.SC_OK, response.getStatusCodeValue());
        assertEquals(input, response.getBody());
    }

    @Test
    public void should_return204_when_updatingSecretForValidSubscriber() {
        HmacSecret secret = new HmacSecret();
        when(updateSubscriptionSecret.get()).thenReturn(uss);
        when(uss.run(secret, "1")).thenReturn(true);

        ResponseEntity response = this.sut.updateSecret(secret, "1");

        assertEquals(HttpStatus.SC_NO_CONTENT, response.getStatusCodeValue());
        assertNull(response.getBody());
    }

    @Test
    public void should_throw404_when_updatingSecretForInvalidSubscriber() {
        HmacSecret secret = new HmacSecret();
        when(updateSubscriptionSecret.get()).thenReturn(uss);
        when(uss.run(secret, "1")).thenReturn(false);

        try {
            this.sut.updateSecret(new HmacSecret(), "1");
            fail("Should not succeed!");
        } catch (AppException e) {
            assertEquals(404, e.getError().getCode());
            assertEquals("Could not update subscription with id 1 because it could not be found", e.getError().getMessage());
            assertEquals("Not found", e.getError().getReason());
        }
        verify(this.auditLog, times(0)).deleteSubscriptionSuccess("1");
    }
}