/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.ddms.model.validators;

import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.utils.OpenApiFileUtils;
import org.junit.Test;
import org.mockito.ArgumentCaptor;

import javax.validation.ConstraintValidatorContext;
import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class OpenApi3SpecValidatorTest {

    @Test
    public void should_failValidation_when_usingInvalidSpecFile() throws Exception {
        ConstraintValidatorContext context = mock(ConstraintValidatorContext.class);
        ArgumentCaptor<String> msgCaptor = ArgumentCaptor.forClass(String.class);
        when(context.buildConstraintViolationWithTemplate(msgCaptor.capture())).
                thenReturn(mock(ConstraintValidatorContext.ConstraintViolationBuilder.class));
        OpenApi3SpecValidator validator = new OpenApi3SpecValidator();

        Ddms model = OpenApiFileUtils.getDmsWithOpenSpec("validations/openapi/invalidOpenApiVersion.json", "testType");

        List<String> validationErrors = msgCaptor.getAllValues();
        assertFalse(validator.isValid(model.getFirst(), context));
        assertTrue(validationErrors.contains("Value '2.0.1' does not match required pattern '3\\.\\d+(\\.\\d+.*)?'"));
    }

    @Test
    public void should_notFailValidation_when_noGetPath() throws Exception {
        ConstraintValidatorContext context = mock(ConstraintValidatorContext.class);
        ArgumentCaptor<String> msgCaptor = ArgumentCaptor.forClass(String.class);
        when(context.buildConstraintViolationWithTemplate(msgCaptor.capture())).
                thenReturn(mock(ConstraintValidatorContext.ConstraintViolationBuilder.class));
        OpenApi3SpecValidator validator = new OpenApi3SpecValidator();

        Ddms model = OpenApiFileUtils.getDmsWithOpenSpec("validations/path/noGetMethod.json", "testType");

        assertTrue(validator.isValid(model.getFirst(), context));
    }

    @Test
    public void should_notFailValidation_when_usingValidSpecFile() throws Exception {
        ConstraintValidatorContext context = mock(ConstraintValidatorContext.class);
        ArgumentCaptor<String> msgCaptor = ArgumentCaptor.forClass(String.class);
        when(context.buildConstraintViolationWithTemplate(msgCaptor.capture())).
                thenReturn(mock(ConstraintValidatorContext.ConstraintViolationBuilder.class));
        OpenApi3SpecValidator validator = new OpenApi3SpecValidator();

        Ddms model = OpenApiFileUtils.getDmsWithOpenSpec("validSubsurfaceDdms.json", "testType");

        List<String> validationErrors = msgCaptor.getAllValues();
        assertTrue(validationErrors.toString(), validationErrors.isEmpty());
        assertTrue(validator.isValid(model.getFirst(), context));
    }

    @Test
    public void should_failValidation_when_serversMissing() throws Exception {
        ConstraintValidatorContext context = mock(ConstraintValidatorContext.class);
        ArgumentCaptor<String> msgCaptor = ArgumentCaptor.forClass(String.class);
        when(context.buildConstraintViolationWithTemplate(msgCaptor.capture())).
                thenReturn(mock(ConstraintValidatorContext.ConstraintViolationBuilder.class));
        OpenApi3SpecValidator validator = new OpenApi3SpecValidator();

        Ddms model = OpenApiFileUtils.getDmsWithOpenSpec("validations/servers/missingServers.json", "testType");

        assertFalse(validator.isValid(model.getFirst(), context));
        assertEquals("Invalid servers field on open api spec, There must be one server only.", msgCaptor.getValue());
    }

    @Test
    public void should_failValidation_when_multipleServersRegistered() throws Exception {
        ConstraintValidatorContext context = mock(ConstraintValidatorContext.class);
        ArgumentCaptor<String> msgCaptor = ArgumentCaptor.forClass(String.class);
        when(context.buildConstraintViolationWithTemplate(msgCaptor.capture())).
                thenReturn(mock(ConstraintValidatorContext.ConstraintViolationBuilder.class));
        OpenApi3SpecValidator validator = new OpenApi3SpecValidator();

        Ddms model = OpenApiFileUtils.getDmsWithOpenSpec("validations/servers/multipleServers.json", "testType");

        assertFalse(validator.isValid(model.getFirst(), context));
        assertEquals("Invalid servers field on open api spec, There must be one server only.", msgCaptor.getValue());
    }

}