/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber.services;

import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.logging.AuditLog;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ISubscriptionRepository;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.*;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class UpdateSubscriptionTest {
    @Mock
    private ISubscriptionRepository subscriptionRepository;
    @Mock
    private ChallengeResponseCheck challengeResponseCheck;
    @Mock
    private AuditLog auditLogger;

    @InjectMocks
    private UpdateSubscriptionSecret sut;

    @Test
    public void should_throw400_when_failsChallenge() {
        String id = "id_1";
        Subscription subscription = new Subscription();
        subscription.setId(id);
        HmacSecret newSecret = new HmacSecret();
        newSecret.setValue("newValue");
        when(subscriptionRepository.get(id)).thenReturn(subscription);
        when(challengeResponseCheck.runUpdate(subscription, newSecret)).thenReturn(false);

        try {
            sut.run(newSecret, id);
            fail("Expected error");
        } catch (AppException e) {
            assertEquals(400, e.getError().getCode());
            assertTrue(e.getError().getMessage().startsWith("Failed challenge response check to"));
            assertEquals("Bad Request", e.getError().getReason());
        }
    }

    @Test
    public void should_return_false_when_patchReturnsFail() {
        String id = "id_1";
        Subscription subscription = new Subscription();
        subscription.setId(id);
        HmacSecret newSecret = new HmacSecret();
        newSecret.setValue("newValue");
        when(subscriptionRepository.get(id)).thenReturn(subscription);
        when(subscriptionRepository.patch(subscription, newSecret)).thenReturn(false);
        when(challengeResponseCheck.runUpdate(subscription, newSecret)).thenReturn(true);
        assertFalse(sut.run(newSecret, id));
    }

    @Test
    public void should_returnTrueWhenUpdateSucceeds() {
        String id = "id_1";
        HmacSecret newSecret = new HmacSecret();
        newSecret.setValue("newValue");
        Subscription subscription = new Subscription();
        subscription.setId(id);
        when(subscriptionRepository.get(id)).thenReturn(subscription);
        when(challengeResponseCheck.runUpdate(subscription, newSecret)).thenReturn(true);
        when(subscriptionRepository.patch(subscription, newSecret)).thenReturn(true);

        assertTrue(sut.run(newSecret, subscription.getId()));
        verify(this.auditLogger).updateSubscriptionSecretSuccess(any());
        verify(this.subscriptionRepository).patch(subscription, newSecret);
    }
}
